package com.xinyou.flutter.flutter_baidu_imageclassify;

import android.os.Handler;
import android.os.Looper;

import androidx.annotation.NonNull;
import com.baidu.aip.imageclassify.AipImageClassify;
import org.json.JSONException;
import org.json.JSONObject;
import java.util.HashMap;
import io.flutter.plugin.common.EventChannel;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;
import io.flutter.plugin.common.PluginRegistry.Registrar;

/** FlutterBaiduImageclassifyPlugin */
public class FlutterBaiduImageclassifyPlugin implements MethodCallHandler, EventChannel.StreamHandler{

  private static final String CHANNEL_METHOD_IMAGECLASSIFY = "flutter_baidu_imageclassify";
  private static final String CHANNEL_STREAM_IMAGECLASSIFY = "flutter_baidu_imageclassify_stream";

  private EventChannel.EventSink mEventSink = null;
  private AipImageClassify mClient=null;

  FlutterBaiduImageclassifyPlugin() {}

  /**
   * 注册组件
   */
  public static void registerWith(Registrar registrar) {
    FlutterBaiduImageclassifyPlugin plugin = new FlutterBaiduImageclassifyPlugin();
    final MethodChannel channel = new MethodChannel(registrar.messenger(), CHANNEL_METHOD_IMAGECLASSIFY);
    channel.setMethodCallHandler(plugin);
    final EventChannel eventChannel = new EventChannel(registrar.messenger(), CHANNEL_STREAM_IMAGECLASSIFY);
    eventChannel.setStreamHandler(plugin);
  }

  @Override
  public void onListen(Object o, EventChannel.EventSink eventSink) {
    mEventSink = eventSink;
  }

  @Override
  public void onCancel(Object arguments) {

  }

  @Override
  public void onMethodCall(@NonNull MethodCall call, @NonNull Result result) {
    if ("classify".equals(call.method)) {
      if(call.argument("path")!=null){
          classify((String) call.argument("path"),(String) call.argument("APP_ID"),(String) call.argument("API_KEY"),
                  (String) call.argument("SECRET_KEY"),(String) call.argument("CONNECT_TIMEOUT"),
                  (String) call.argument("SOCKET_TIMEOUT"),(String) call.argument("TOP_NUM"),(String) call.argument("BAIKE_NUM"));
      }
    }else if (("getPlatformVersion").equals(call.method)) {
      result.success("Android " + android.os.Build.VERSION.RELEASE);
    }else {
      result.notImplemented();
    }
  }

  /**
   * 检测指定路径的图片
   * @param path
   * @return
   * @throws JSONException
   */
  public void classify(final String path,String APP_ID,String API_KEY,String SECRET_KEY,String CONNECT_TIMEOUT,String SOCKET_TIMEOUT,String TOP_NUM,String BAIKE_NUM){
    if(mClient==null){
      mClient = new AipImageClassify(APP_ID, API_KEY, SECRET_KEY);
      mClient.setConnectionTimeoutInMillis(Integer.valueOf(CONNECT_TIMEOUT));
      mClient.setSocketTimeoutInMillis(Integer.valueOf(SOCKET_TIMEOUT));
    }

    // 传入可选参数调用接口
    final HashMap<String, String> options = new HashMap<String, String>();
    options.put("top_num", TOP_NUM);
    options.put("baike_num", BAIKE_NUM);

    System.out.println(path);

    new Thread(new Runnable(){
      @Override
      public void run() {
        try {
          final JSONObject res = mClient.animalDetect(path, options);
          System.out.println(res.toString());
          Handler mainThread = new Handler(Looper.getMainLooper());
          mainThread.post(new Runnable() {
            @Override
            public void run() {
              if(mEventSink!=null) mEventSink.success(res.toString());
            }
          });
        } catch (Exception e) {
          e.printStackTrace();
          Handler mainThread = new Handler(Looper.getMainLooper());
          mainThread.post(new Runnable() {
            @Override
            public void run() {
              if(mEventSink!=null) mEventSink.success("error");
            }
          });
        }
      }
    }).start();
  }
}
